"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from ._version import __title__, __version__
from .sdk import *
from .sdkconfiguration import *
from .models import *


VERSION: str = __version__



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutpricecreate import CheckoutPriceCreate, CheckoutPriceCreateTypedDict
from .checkoutproductcreate import CheckoutProductCreate, CheckoutProductCreateTypedDict
from typing import Union
from typing_extensions import TypeAliasType


CheckoutCreateTypedDict = TypeAliasType(
    "CheckoutCreateTypedDict",
    Union[CheckoutProductCreateTypedDict, CheckoutPriceCreateTypedDict],
)


CheckoutCreate = TypeAliasType(
    "CheckoutCreate", Union[CheckoutProductCreate, CheckoutPriceCreate]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class MetricType(str, Enum):
    SCALAR = "scalar"
    CURRENCY = "currency"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customersortproperty import CustomerSortProperty
from .listresource_customer_ import ListResourceCustomer, ListResourceCustomerTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CustomersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "CustomersListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


CustomersListQueryParamOrganizationIDFilter = TypeAliasType(
    "CustomersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


class CustomersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[CustomersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    query: NotRequired[Nullable[str]]
    r"""Filter by name or email."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[CustomerSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class CustomersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[CustomersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by name or email."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[CustomerSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["organization_id", "query", "page", "limit", "sorting"]
        nullable_fields = ["organization_id", "query", "sorting"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomersListResponseTypedDict(TypedDict):
    result: ListResourceCustomerTypedDict


class CustomersListResponse(BaseModel):
    next: Callable[[], Optional[CustomersListResponse]]

    result: ListResourceCustomer



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfield import CustomField, CustomFieldTypedDict
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class AttachedCustomFieldTypedDict(TypedDict):
    r"""Schema of a custom field attached to a resource."""

    custom_field_id: str
    r"""ID of the custom field."""
    custom_field: CustomFieldTypedDict
    order: int
    r"""Order of the custom field in the resource."""
    required: bool
    r"""Whether the value is required for this custom field."""


class AttachedCustomField(BaseModel):
    r"""Schema of a custom field attached to a resource."""

    custom_field_id: str
    r"""ID of the custom field."""

    custom_field: CustomField

    order: int
    r"""Order of the custom field in the resource."""

    required: bool
    r"""Whether the value is required for this custom field."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .taxidformat import TaxIDFormat
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Union
from typing_extensions import TypeAliasType, TypedDict


OrderCustomerMetadataTypedDict = TypeAliasType(
    "OrderCustomerMetadataTypedDict", Union[str, int, bool]
)


OrderCustomerMetadata = TypeAliasType("OrderCustomerMetadata", Union[str, int, bool])


OrderCustomerTaxIDTypedDict = TypeAliasType(
    "OrderCustomerTaxIDTypedDict", Union[str, TaxIDFormat]
)


OrderCustomerTaxID = TypeAliasType("OrderCustomerTaxID", Union[str, TaxIDFormat])


class OrderCustomerTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    metadata: Dict[str, OrderCustomerMetadataTypedDict]
    email: str
    email_verified: bool
    name: Nullable[str]
    billing_address: Nullable[AddressTypedDict]
    tax_id: Nullable[List[OrderCustomerTaxIDTypedDict]]
    organization_id: str
    avatar_url: str


class OrderCustomer(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    metadata: Dict[str, OrderCustomerMetadata]

    email: str

    email_verified: bool

    name: Nullable[str]

    billing_address: Nullable[Address]

    tax_id: Nullable[List[OrderCustomerTaxID]]

    organization_id: str

    avatar_url: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "name", "billing_address", "tax_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantDiscordPropertiesTypedDict(TypedDict):
    account_id: NotRequired[str]
    guild_id: NotRequired[str]
    role_id: NotRequired[str]


class BenefitGrantDiscordProperties(BaseModel):
    account_id: Optional[str] = None

    guild_id: Optional[str] = None

    role_id: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .discountfixedonceforeverdurationbase import (
    DiscountFixedOnceForeverDurationBase,
    DiscountFixedOnceForeverDurationBaseTypedDict,
)
from .discountfixedrepeatdurationbase import (
    DiscountFixedRepeatDurationBase,
    DiscountFixedRepeatDurationBaseTypedDict,
)
from .discountpercentageonceforeverdurationbase import (
    DiscountPercentageOnceForeverDurationBase,
    DiscountPercentageOnceForeverDurationBaseTypedDict,
)
from .discountpercentagerepeatdurationbase import (
    DiscountPercentageRepeatDurationBase,
    DiscountPercentageRepeatDurationBaseTypedDict,
)
from .product import Product, ProductTypedDict
from .productpricerecurring import ProductPriceRecurring, ProductPriceRecurringTypedDict
from .subscriptioncustomer import SubscriptionCustomer, SubscriptionCustomerTypedDict
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from .subscriptionstatus import SubscriptionStatus
from .subscriptionuser import SubscriptionUser, SubscriptionUserTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


MetadataTypedDict = TypeAliasType("MetadataTypedDict", Union[str, int, bool])


Metadata = TypeAliasType("Metadata", Union[str, int, bool])


class SubscriptionCustomFieldDataTypedDict(TypedDict):
    r"""Key-value object storing custom field values."""


class SubscriptionCustomFieldData(BaseModel):
    r"""Key-value object storing custom field values."""


SubscriptionDiscountTypedDict = TypeAliasType(
    "SubscriptionDiscountTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationBaseTypedDict,
        DiscountFixedOnceForeverDurationBaseTypedDict,
        DiscountPercentageRepeatDurationBaseTypedDict,
        DiscountFixedRepeatDurationBaseTypedDict,
    ],
)


SubscriptionDiscount = TypeAliasType(
    "SubscriptionDiscount",
    Union[
        DiscountPercentageOnceForeverDurationBase,
        DiscountFixedOnceForeverDurationBase,
        DiscountPercentageRepeatDurationBase,
        DiscountFixedRepeatDurationBase,
    ],
)


class SubscriptionTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    amount: Nullable[int]
    currency: Nullable[str]
    recurring_interval: SubscriptionRecurringInterval
    status: SubscriptionStatus
    current_period_start: datetime
    current_period_end: Nullable[datetime]
    cancel_at_period_end: bool
    started_at: Nullable[datetime]
    ended_at: Nullable[datetime]
    customer_id: str
    product_id: str
    price_id: str
    discount_id: Nullable[str]
    checkout_id: Nullable[str]
    metadata: Dict[str, MetadataTypedDict]
    customer: SubscriptionCustomerTypedDict
    user_id: str
    user: SubscriptionUserTypedDict
    product: ProductTypedDict
    r"""A product."""
    price: ProductPriceRecurringTypedDict
    discount: Nullable[SubscriptionDiscountTypedDict]
    custom_field_data: NotRequired[SubscriptionCustomFieldDataTypedDict]
    r"""Key-value object storing custom field values."""


class Subscription(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    amount: Nullable[int]

    currency: Nullable[str]

    recurring_interval: SubscriptionRecurringInterval

    status: SubscriptionStatus

    current_period_start: datetime

    current_period_end: Nullable[datetime]

    cancel_at_period_end: bool

    started_at: Nullable[datetime]

    ended_at: Nullable[datetime]

    customer_id: str

    product_id: str

    price_id: str

    discount_id: Nullable[str]

    checkout_id: Nullable[str]

    metadata: Dict[str, Metadata]

    customer: SubscriptionCustomer

    user_id: Annotated[
        str,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    user: SubscriptionUser

    product: Product
    r"""A product."""

    price: ProductPriceRecurring

    discount: Nullable[SubscriptionDiscount]

    custom_field_data: Optional[SubscriptionCustomFieldData] = None
    r"""Key-value object storing custom field values."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["custom_field_data"]
        nullable_fields = [
            "modified_at",
            "amount",
            "currency",
            "current_period_end",
            "started_at",
            "ended_at",
            "discount_id",
            "checkout_id",
            "discount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class S3DownloadURLTypedDict(TypedDict):
    url: str
    expires_at: datetime
    headers: NotRequired[Dict[str, str]]


class S3DownloadURL(BaseModel):
    url: str

    expires_at: datetime

    headers: Optional[Dict[str, str]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class Scope(str, Enum):
    OPENID = "openid"
    PROFILE = "profile"
    EMAIL = "email"
    USER_READ = "user:read"
    ADMIN = "admin"
    WEB_DEFAULT = "web_default"
    ORGANIZATIONS_READ = "organizations:read"
    ORGANIZATIONS_WRITE = "organizations:write"
    CUSTOM_FIELDS_READ = "custom_fields:read"
    CUSTOM_FIELDS_WRITE = "custom_fields:write"
    DISCOUNTS_READ = "discounts:read"
    DISCOUNTS_WRITE = "discounts:write"
    CHECKOUT_LINKS_READ = "checkout_links:read"
    CHECKOUT_LINKS_WRITE = "checkout_links:write"
    CHECKOUTS_READ = "checkouts:read"
    CHECKOUTS_WRITE = "checkouts:write"
    PRODUCTS_READ = "products:read"
    PRODUCTS_WRITE = "products:write"
    BENEFITS_READ = "benefits:read"
    BENEFITS_WRITE = "benefits:write"
    FILES_READ = "files:read"
    FILES_WRITE = "files:write"
    SUBSCRIPTIONS_READ = "subscriptions:read"
    SUBSCRIPTIONS_WRITE = "subscriptions:write"
    CUSTOMERS_READ = "customers:read"
    CUSTOMERS_WRITE = "customers:write"
    CUSTOMER_SESSIONS_WRITE = "customer_sessions:write"
    ORDERS_READ = "orders:read"
    METRICS_READ = "metrics:read"
    WEBHOOKS_READ = "webhooks:read"
    WEBHOOKS_WRITE = "webhooks:write"
    EXTERNAL_ORGANIZATIONS_READ = "external_organizations:read"
    LICENSE_KEYS_READ = "license_keys:read"
    LICENSE_KEYS_WRITE = "license_keys:write"
    REPOSITORIES_READ = "repositories:read"
    REPOSITORIES_WRITE = "repositories:write"
    ISSUES_READ = "issues:read"
    ISSUES_WRITE = "issues:write"
    CUSTOMER_PORTAL_READ = "customer_portal:read"
    CUSTOMER_PORTAL_WRITE = "customer_portal:write"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metric import Metric, MetricTypedDict
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MetricsTypedDict(TypedDict):
    orders: MetricTypedDict
    r"""Information about a metric."""
    revenue: MetricTypedDict
    r"""Information about a metric."""
    average_order_value: MetricTypedDict
    r"""Information about a metric."""
    one_time_products: MetricTypedDict
    r"""Information about a metric."""
    one_time_products_revenue: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions_revenue: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions_revenue: MetricTypedDict
    r"""Information about a metric."""
    active_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    monthly_recurring_revenue: MetricTypedDict
    r"""Information about a metric."""


class Metrics(BaseModel):
    orders: Metric
    r"""Information about a metric."""

    revenue: Metric
    r"""Information about a metric."""

    average_order_value: Metric
    r"""Information about a metric."""

    one_time_products: Metric
    r"""Information about a metric."""

    one_time_products_revenue: Metric
    r"""Information about a metric."""

    new_subscriptions: Metric
    r"""Information about a metric."""

    new_subscriptions_revenue: Metric
    r"""Information about a metric."""

    renewed_subscriptions: Metric
    r"""Information about a metric."""

    renewed_subscriptions_revenue: Metric
    r"""Information about a metric."""

    active_subscriptions: Metric
    r"""Information about a metric."""

    monthly_recurring_revenue: Metric
    r"""Information about a metric."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SubscriptionSortProperty(str, Enum):
    CUSTOMER = "customer"
    MINUS_CUSTOMER = "-customer"
    STATUS = "status"
    MINUS_STATUS = "-status"
    STARTED_AT = "started_at"
    MINUS_STARTED_AT = "-started_at"
    CURRENT_PERIOD_END = "current_period_end"
    MINUS_CURRENT_PERIOD_END = "-current_period_end"
    AMOUNT = "amount"
    MINUS_AMOUNT = "-amount"
    PRODUCT = "product"
    MINUS_PRODUCT = "-product"
    DISCOUNT = "discount"
    MINUS_DISCOUNT = "-discount"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SubscriptionRecurringInterval(str, Enum):
    MONTH = "month"
    YEAR = "year"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CustomerSessionCreateTypedDict(TypedDict):
    r"""Schema for creating a customer session."""

    customer_id: str
    r"""ID of the customer to create a session for."""


class CustomerSessionCreate(BaseModel):
    r"""Schema for creating a customer session."""

    customer_id: str
    r"""ID of the customer to create a session for."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerbenefitgrantads import (
    CustomerBenefitGrantAds,
    CustomerBenefitGrantAdsTypedDict,
)
from .customerbenefitgrantcustom import (
    CustomerBenefitGrantCustom,
    CustomerBenefitGrantCustomTypedDict,
)
from .customerbenefitgrantdiscord import (
    CustomerBenefitGrantDiscord,
    CustomerBenefitGrantDiscordTypedDict,
)
from .customerbenefitgrantdownloadables import (
    CustomerBenefitGrantDownloadables,
    CustomerBenefitGrantDownloadablesTypedDict,
)
from .customerbenefitgrantgithubrepository import (
    CustomerBenefitGrantGitHubRepository,
    CustomerBenefitGrantGitHubRepositoryTypedDict,
)
from .customerbenefitgrantlicensekeys import (
    CustomerBenefitGrantLicenseKeys,
    CustomerBenefitGrantLicenseKeysTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


CustomerBenefitGrantTypedDict = TypeAliasType(
    "CustomerBenefitGrantTypedDict",
    Union[
        CustomerBenefitGrantDiscordTypedDict,
        CustomerBenefitGrantGitHubRepositoryTypedDict,
        CustomerBenefitGrantDownloadablesTypedDict,
        CustomerBenefitGrantLicenseKeysTypedDict,
        CustomerBenefitGrantAdsTypedDict,
        CustomerBenefitGrantCustomTypedDict,
    ],
)


CustomerBenefitGrant = TypeAliasType(
    "CustomerBenefitGrant",
    Union[
        CustomerBenefitGrantDiscord,
        CustomerBenefitGrantGitHubRepository,
        CustomerBenefitGrantDownloadables,
        CustomerBenefitGrantLicenseKeys,
        CustomerBenefitGrantAds,
        CustomerBenefitGrantCustom,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitcustomsubscriber import (
    BenefitCustomSubscriber,
    BenefitCustomSubscriberTypedDict,
)
from .benefitgrantcustomproperties import (
    BenefitGrantCustomProperties,
    BenefitGrantCustomPropertiesTypedDict,
)
from .customerportalcustomer import (
    CustomerPortalCustomer,
    CustomerPortalCustomerTypedDict,
)
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class CustomerBenefitGrantCustomTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    granted_at: Nullable[datetime]
    revoked_at: Nullable[datetime]
    customer_id: str
    benefit_id: str
    subscription_id: Nullable[str]
    order_id: Nullable[str]
    is_granted: bool
    is_revoked: bool
    customer: CustomerPortalCustomerTypedDict
    benefit: BenefitCustomSubscriberTypedDict
    properties: BenefitGrantCustomPropertiesTypedDict


class CustomerBenefitGrantCustom(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    granted_at: Nullable[datetime]

    revoked_at: Nullable[datetime]

    customer_id: str

    benefit_id: str

    subscription_id: Nullable[str]

    order_id: Nullable[str]

    is_granted: bool

    is_revoked: bool

    customer: CustomerPortalCustomer

    benefit: BenefitCustomSubscriber

    properties: BenefitGrantCustomProperties

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "granted_at",
            "revoked_at",
            "subscription_id",
            "order_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.custom import Custom
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from typing import Any, Mapping, Optional, Union, cast
from typing_extensions import deprecated


class Checkouts(BaseSDK):
    custom: Custom

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.custom = Custom(self.sdk_configuration)

    @deprecated(
        "warning: ** DEPRECATED ** - This API is deprecated. We recommend you to use the new custom checkout API, which is more flexible and powerful. Please refer to the documentation for more information.. Use create instead."
    )
    def create(
        self,
        *,
        request: Union[
            models.CheckoutLegacyCreate, models.CheckoutLegacyCreateTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CheckoutLegacy:
        r"""Create Checkout

        Create a checkout session.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CheckoutLegacyCreate)
        request = cast(models.CheckoutLegacyCreate, request)

        req = self._build_request(
            method="POST",
            path="/v1/checkouts/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CheckoutLegacyCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="checkouts:create",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CheckoutLegacy)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    @deprecated(
        "warning: ** DEPRECATED ** - This API is deprecated. We recommend you to use the new custom checkout API, which is more flexible and powerful. Please refer to the documentation for more information.. Use create instead."
    )
    async def create_async(
        self,
        *,
        request: Union[
            models.CheckoutLegacyCreate, models.CheckoutLegacyCreateTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CheckoutLegacy:
        r"""Create Checkout

        Create a checkout session.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CheckoutLegacyCreate)
        request = cast(models.CheckoutLegacyCreate, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/checkouts/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CheckoutLegacyCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="checkouts:create",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CheckoutLegacy)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    @deprecated(
        "warning: ** DEPRECATED ** - This API is deprecated. We recommend you to use the new custom checkout API, which is more flexible and powerful. Please refer to the documentation for more information.."
    )
    def get(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CheckoutLegacy:
        r"""Get Checkout

        Get an active checkout session by ID.

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.CheckoutsGetRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/checkouts/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="checkouts:get",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CheckoutLegacy)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    @deprecated(
        "warning: ** DEPRECATED ** - This API is deprecated. We recommend you to use the new custom checkout API, which is more flexible and powerful. Please refer to the documentation for more information.."
    )
    async def get_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CheckoutLegacy:
        r"""Get Checkout

        Get an active checkout session by ID.

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.CheckoutsGetRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/checkouts/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="checkouts:get",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CheckoutLegacy)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.HTTPValidationErrorData)
            raise models.HTTPValidationError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutlink import CheckoutLink, CheckoutLinkTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCheckoutLinkTypedDict(TypedDict):
    items: List[CheckoutLinkTypedDict]
    pagination: PaginationTypedDict


class ListResourceCheckoutLink(BaseModel):
    items: List[CheckoutLink]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .discountfixedonceforeverduration import (
    DiscountFixedOnceForeverDuration,
    DiscountFixedOnceForeverDurationTypedDict,
)
from .discountfixedrepeatduration import (
    DiscountFixedRepeatDuration,
    DiscountFixedRepeatDurationTypedDict,
)
from .discountpercentageonceforeverduration import (
    DiscountPercentageOnceForeverDuration,
    DiscountPercentageOnceForeverDurationTypedDict,
)
from .discountpercentagerepeatduration import (
    DiscountPercentageRepeatDuration,
    DiscountPercentageRepeatDurationTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


DiscountTypedDict = TypeAliasType(
    "DiscountTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationTypedDict,
        DiscountFixedOnceForeverDurationTypedDict,
        DiscountPercentageRepeatDurationTypedDict,
        DiscountFixedRepeatDurationTypedDict,
    ],
)


Discount = TypeAliasType(
    "Discount",
    Union[
        DiscountPercentageOnceForeverDuration,
        DiscountFixedOnceForeverDuration,
        DiscountPercentageRepeatDuration,
        DiscountFixedRepeatDuration,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .downloadablefileread import DownloadableFileRead, DownloadableFileReadTypedDict
from .fileuploadcompleted import FileUploadCompleted, FileUploadCompletedTypedDict
from .organizationavatarfileread import (
    OrganizationAvatarFileRead,
    OrganizationAvatarFileReadTypedDict,
)
from .productmediafileread import ProductMediaFileRead, ProductMediaFileReadTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import (
    FieldMetadata,
    PathParamMetadata,
    RequestMetadata,
    get_discriminator,
)
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class FilesUploadedRequestTypedDict(TypedDict):
    id: str
    r"""The file ID."""
    file_upload_completed: FileUploadCompletedTypedDict


class FilesUploadedRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The file ID."""

    file_upload_completed: Annotated[
        FileUploadCompleted,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


FilesUploadedResponseFilesUploadedTypedDict = TypeAliasType(
    "FilesUploadedResponseFilesUploadedTypedDict",
    Union[
        DownloadableFileReadTypedDict,
        ProductMediaFileReadTypedDict,
        OrganizationAvatarFileReadTypedDict,
    ],
)
r"""File upload completed."""


FilesUploadedResponseFilesUploaded = Annotated[
    Union[
        Annotated[DownloadableFileRead, Tag("downloadable")],
        Annotated[ProductMediaFileRead, Tag("product_media")],
        Annotated[OrganizationAvatarFileRead, Tag("organization_avatar")],
    ],
    Discriminator(lambda m: get_discriminator(m, "service", "service")),
]
r"""File upload completed."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutlinkproduct import CheckoutLinkProduct, CheckoutLinkProductTypedDict
from .discountfixedonceforeverdurationbase import (
    DiscountFixedOnceForeverDurationBase,
    DiscountFixedOnceForeverDurationBaseTypedDict,
)
from .discountfixedrepeatdurationbase import (
    DiscountFixedRepeatDurationBase,
    DiscountFixedRepeatDurationBaseTypedDict,
)
from .discountpercentageonceforeverdurationbase import (
    DiscountPercentageOnceForeverDurationBase,
    DiscountPercentageOnceForeverDurationBaseTypedDict,
)
from .discountpercentagerepeatdurationbase import (
    DiscountPercentageRepeatDurationBase,
    DiscountPercentageRepeatDurationBaseTypedDict,
)
from .paymentprocessor import PaymentProcessor
from .productprice import ProductPrice, ProductPriceTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Union
from typing_extensions import TypeAliasType, TypedDict


CheckoutLinkMetadataTypedDict = TypeAliasType(
    "CheckoutLinkMetadataTypedDict", Union[str, int, bool]
)


CheckoutLinkMetadata = TypeAliasType("CheckoutLinkMetadata", Union[str, int, bool])


CheckoutLinkDiscountTypedDict = TypeAliasType(
    "CheckoutLinkDiscountTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationBaseTypedDict,
        DiscountFixedOnceForeverDurationBaseTypedDict,
        DiscountPercentageRepeatDurationBaseTypedDict,
        DiscountFixedRepeatDurationBaseTypedDict,
    ],
)


CheckoutLinkDiscount = TypeAliasType(
    "CheckoutLinkDiscount",
    Union[
        DiscountPercentageOnceForeverDurationBase,
        DiscountFixedOnceForeverDurationBase,
        DiscountPercentageRepeatDurationBase,
        DiscountFixedRepeatDurationBase,
    ],
)


class CheckoutLinkTypedDict(TypedDict):
    r"""Checkout link data."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    metadata: Dict[str, CheckoutLinkMetadataTypedDict]
    payment_processor: PaymentProcessor
    client_secret: str
    r"""Client secret used to access the checkout link."""
    success_url: Nullable[str]
    r"""URL where the customer will be redirected after a successful payment."""
    label: Nullable[str]
    r"""Optional label to distinguish links internally"""
    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    product_id: str
    r"""ID of the product to checkout."""
    product_price_id: Nullable[str]
    r"""ID of the product price to checkout. First available price will be selected unless an explicit price ID is set."""
    discount_id: Nullable[str]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""
    product: CheckoutLinkProductTypedDict
    r"""Product data for a checkout link."""
    product_price: Nullable[ProductPriceTypedDict]
    discount: Nullable[CheckoutLinkDiscountTypedDict]
    url: str


class CheckoutLink(BaseModel):
    r"""Checkout link data."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    metadata: Dict[str, CheckoutLinkMetadata]

    payment_processor: PaymentProcessor

    client_secret: str
    r"""Client secret used to access the checkout link."""

    success_url: Nullable[str]
    r"""URL where the customer will be redirected after a successful payment."""

    label: Nullable[str]
    r"""Optional label to distinguish links internally"""

    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    product_id: str
    r"""ID of the product to checkout."""

    product_price_id: Nullable[str]
    r"""ID of the product price to checkout. First available price will be selected unless an explicit price ID is set."""

    discount_id: Nullable[str]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""

    product: CheckoutLinkProduct
    r"""Product data for a checkout link."""

    product_price: Nullable[ProductPrice]

    discount: Nullable[CheckoutLinkDiscount]

    url: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "success_url",
            "label",
            "product_price_id",
            "discount_id",
            "product_price",
            "discount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .interval import Interval
from .productpricetype import ProductPriceType
from datetime import date
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


MetricsGetQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "MetricsGetQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


MetricsGetQueryParamOrganizationIDFilter = TypeAliasType(
    "MetricsGetQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


MetricsGetQueryParamProductIDFilterTypedDict = TypeAliasType(
    "MetricsGetQueryParamProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


MetricsGetQueryParamProductIDFilter = TypeAliasType(
    "MetricsGetQueryParamProductIDFilter", Union[str, List[str]]
)
r"""Filter by product ID."""


QueryParamProductPriceTypeFilterTypedDict = TypeAliasType(
    "QueryParamProductPriceTypeFilterTypedDict",
    Union[ProductPriceType, List[ProductPriceType]],
)
r"""Filter by product price type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


QueryParamProductPriceTypeFilter = TypeAliasType(
    "QueryParamProductPriceTypeFilter", Union[ProductPriceType, List[ProductPriceType]]
)
r"""Filter by product price type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


class MetricsGetRequestTypedDict(TypedDict):
    start_date: date
    r"""Start date."""
    end_date: date
    r"""End date."""
    interval: Interval
    r"""Interval between two timestamps."""
    organization_id: NotRequired[
        Nullable[MetricsGetQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    product_id: NotRequired[Nullable[MetricsGetQueryParamProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    product_price_type: NotRequired[Nullable[QueryParamProductPriceTypeFilterTypedDict]]
    r"""Filter by product price type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


class MetricsGetRequest(BaseModel):
    start_date: Annotated[
        date, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Start date."""

    end_date: Annotated[
        date, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""End date."""

    interval: Annotated[
        Interval, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Interval between two timestamps."""

    organization_id: Annotated[
        OptionalNullable[MetricsGetQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[MetricsGetQueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    product_price_type: Annotated[
        OptionalNullable[QueryParamProductPriceTypeFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product price type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["organization_id", "product_id", "product_price_type"]
        nullable_fields = ["organization_id", "product_id", "product_price_type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productonetimecreate import ProductOneTimeCreate, ProductOneTimeCreateTypedDict
from .productrecurringcreate import (
    ProductRecurringCreate,
    ProductRecurringCreateTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ProductCreateTypedDict = TypeAliasType(
    "ProductCreateTypedDict",
    Union[ProductRecurringCreateTypedDict, ProductOneTimeCreateTypedDict],
)


ProductCreate = TypeAliasType(
    "ProductCreate", Union[ProductRecurringCreate, ProductOneTimeCreate]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequestTypedDict(
    TypedDict
):
    client_id: str
    client_secret: str
    code: str
    redirect_uri: str
    grant_type: Literal["authorization_code"]


class Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequest(BaseModel):
    client_id: Annotated[str, FieldMetadata(form=True)]

    client_secret: Annotated[str, FieldMetadata(form=True)]

    code: Annotated[str, FieldMetadata(form=True)]

    redirect_uri: Annotated[str, FieldMetadata(form=True)]

    GRANT_TYPE: Annotated[
        Annotated[
            Literal["authorization_code"],
            AfterValidator(validate_const("authorization_code")),
        ],
        pydantic.Field(alias="grant_type"),
        FieldMetadata(form=True),
    ] = "authorization_code"



import base64
from typing import Any, Dict, Union

from pydantic import Discriminator, Tag, TypeAdapter
from standardwebhooks.webhooks import Webhook
from standardwebhooks.webhooks import (
    WebhookVerificationError as _WebhookVerificationError,
)
from typing_extensions import Annotated

from polar_sdk.models import (
    WebhookBenefitCreatedPayload,
    WebhookBenefitGrantCreatedPayload,
    WebhookBenefitGrantRevokedPayload,
    WebhookBenefitGrantUpdatedPayload,
    WebhookBenefitUpdatedPayload,
    WebhookCheckoutCreatedPayload,
    WebhookCheckoutUpdatedPayload,
    WebhookOrderCreatedPayload,
    WebhookOrganizationUpdatedPayload,
    WebhookPledgeCreatedPayload,
    WebhookPledgeUpdatedPayload,
    WebhookProductCreatedPayload,
    WebhookProductUpdatedPayload,
    WebhookSubscriptionActivePayload,
    WebhookSubscriptionCanceledPayload,
    WebhookSubscriptionCreatedPayload,
    WebhookSubscriptionRevokedPayload,
    WebhookSubscriptionUpdatedPayload,
)


def _get_discriminator(v: Any) -> Union[str, None]:
    if isinstance(v, dict):
        return v.get("type")
    return getattr(v, "type", None)


WebhoookPayload = Annotated[
    Union[
        Annotated[WebhookBenefitCreatedPayload, Tag("benefit.created")],
        Annotated[WebhookBenefitGrantCreatedPayload, Tag("benefit_grant.created")],
        Annotated[WebhookBenefitGrantRevokedPayload, Tag("benefit_grant.revoked")],
        Annotated[WebhookBenefitGrantUpdatedPayload, Tag("benefit_grant.updated")],
        Annotated[WebhookBenefitUpdatedPayload, Tag("benefit.updated")],
        Annotated[WebhookCheckoutCreatedPayload, Tag("checkout.created")],
        Annotated[WebhookCheckoutUpdatedPayload, Tag("checkout.updated")],
        Annotated[WebhookOrderCreatedPayload, Tag("order.created")],
        Annotated[WebhookOrganizationUpdatedPayload, Tag("organization.updated")],
        Annotated[WebhookPledgeCreatedPayload, Tag("pledge.created")],
        Annotated[WebhookPledgeUpdatedPayload, Tag("pledge.updated")],
        Annotated[WebhookProductCreatedPayload, Tag("product.created")],
        Annotated[WebhookProductUpdatedPayload, Tag("product.updated")],
        Annotated[WebhookSubscriptionActivePayload, Tag("subscription.active")],
        Annotated[WebhookSubscriptionCanceledPayload, Tag("subscription.canceled")],
        Annotated[WebhookSubscriptionCreatedPayload, Tag("subscription.created")],
        Annotated[WebhookSubscriptionRevokedPayload, Tag("subscription.revoked")],
        Annotated[WebhookSubscriptionUpdatedPayload, Tag("subscription.updated")],
    ],
    Discriminator(_get_discriminator),
]

WebhookPayloadAdapter: TypeAdapter[WebhoookPayload] = TypeAdapter(WebhoookPayload)


class WebhookVerificationError(_WebhookVerificationError):
    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(message)


def validate_event(
    body: Union[str, bytes], headers: Dict[str, str], secret: str
) -> WebhoookPayload:
    base64_secret = base64.b64encode(secret.encode()).decode()
    webhook = Webhook(base64_secret)

    try:
        data = webhook.verify(body, headers)
    except _WebhookVerificationError as e:
        raise WebhookVerificationError(str(e)) from e

    return WebhookPayloadAdapter.validate_python(data)


__all__ = [
    "WebhookVerificationError",
    "WebhoookPayload",
    "validate_event",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .attachedcustomfieldcreate import (
    AttachedCustomFieldCreate,
    AttachedCustomFieldCreateTypedDict,
)
from .productpricerecurringfixedcreate import (
    ProductPriceRecurringFixedCreate,
    ProductPriceRecurringFixedCreateTypedDict,
)
from .productpricerecurringfreecreate import (
    ProductPriceRecurringFreeCreate,
    ProductPriceRecurringFreeCreateTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ProductRecurringCreateMetadataTypedDict = TypeAliasType(
    "ProductRecurringCreateMetadataTypedDict", Union[str, int, bool]
)


ProductRecurringCreateMetadata = TypeAliasType(
    "ProductRecurringCreateMetadata", Union[str, int, bool]
)


ProductRecurringCreatePricesTypedDict = TypeAliasType(
    "ProductRecurringCreatePricesTypedDict",
    Union[
        List[ProductPriceRecurringFixedCreateTypedDict],
        List[ProductPriceRecurringFreeCreateTypedDict],
    ],
)
r"""List of available prices for this product."""


ProductRecurringCreatePrices = TypeAliasType(
    "ProductRecurringCreatePrices",
    Union[
        List[ProductPriceRecurringFixedCreate], List[ProductPriceRecurringFreeCreate]
    ],
)
r"""List of available prices for this product."""


class ProductRecurringCreateTypedDict(TypedDict):
    r"""Schema to create a recurring product, i.e. a subscription."""

    name: str
    r"""The name of the product."""
    prices: ProductRecurringCreatePricesTypedDict
    r"""List of available prices for this product."""
    metadata: NotRequired[Dict[str, ProductRecurringCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    description: NotRequired[Nullable[str]]
    r"""The description of the product."""
    medias: NotRequired[Nullable[List[str]]]
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""
    attached_custom_fields: NotRequired[List[AttachedCustomFieldCreateTypedDict]]
    r"""List of custom fields to attach."""
    organization_id: NotRequired[Nullable[str]]
    r"""The ID of the organization owning the product. **Required unless you use an organization token.**"""


class ProductRecurringCreate(BaseModel):
    r"""Schema to create a recurring product, i.e. a subscription."""

    name: str
    r"""The name of the product."""

    prices: ProductRecurringCreatePrices
    r"""List of available prices for this product."""

    metadata: Optional[Dict[str, ProductRecurringCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    description: OptionalNullable[str] = UNSET
    r"""The description of the product."""

    medias: OptionalNullable[List[str]] = UNSET
    r"""List of file IDs. Each one must be on the same organization as the product, of type `product_media` and correctly uploaded."""

    attached_custom_fields: Optional[List[AttachedCustomFieldCreate]] = None
    r"""List of custom fields to attach."""

    organization_id: OptionalNullable[str] = UNSET
    r"""The ID of the organization owning the product. **Required unless you use an organization token.**"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "description",
            "medias",
            "attached_custom_fields",
            "organization_id",
        ]
        nullable_fields = ["description", "medias", "organization_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerbenefitgrantupdate import (
    CustomerBenefitGrantUpdate,
    CustomerBenefitGrantUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class CustomerPortalBenefitGrantsUpdateRequestTypedDict(TypedDict):
    id: str
    r"""The benefit grant ID."""
    customer_benefit_grant_update: CustomerBenefitGrantUpdateTypedDict


class CustomerPortalBenefitGrantsUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The benefit grant ID."""

    customer_benefit_grant_update: Annotated[
        CustomerBenefitGrantUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class AssigneeTypedDict(TypedDict):
    id: int
    login: str
    html_url: str
    avatar_url: str


class Assignee(BaseModel):
    id: int

    login: str

    html_url: str

    avatar_url: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class CheckoutUpdatePublicCustomFieldDataTypedDict(TypedDict):
    pass


class CheckoutUpdatePublicCustomFieldData(BaseModel):
    pass


class CheckoutUpdatePublicTypedDict(TypedDict):
    r"""Update an existing checkout session using the client secret."""

    custom_field_data: NotRequired[
        Nullable[CheckoutUpdatePublicCustomFieldDataTypedDict]
    ]
    r"""Key-value object storing custom field values."""
    product_price_id: NotRequired[Nullable[str]]
    r"""ID of the product price to checkout. Must correspond to a price linked to the same product."""
    amount: NotRequired[Nullable[int]]
    customer_name: NotRequired[Nullable[str]]
    customer_email: NotRequired[Nullable[str]]
    customer_billing_address: NotRequired[Nullable[AddressTypedDict]]
    customer_tax_id: NotRequired[Nullable[str]]
    discount_code: NotRequired[Nullable[str]]
    r"""Discount code to apply to the checkout."""


class CheckoutUpdatePublic(BaseModel):
    r"""Update an existing checkout session using the client secret."""

    custom_field_data: OptionalNullable[CheckoutUpdatePublicCustomFieldData] = UNSET
    r"""Key-value object storing custom field values."""

    product_price_id: OptionalNullable[str] = UNSET
    r"""ID of the product price to checkout. Must correspond to a price linked to the same product."""

    amount: OptionalNullable[int] = UNSET

    customer_name: OptionalNullable[str] = UNSET

    customer_email: OptionalNullable[str] = UNSET

    customer_billing_address: OptionalNullable[Address] = UNSET

    customer_tax_id: OptionalNullable[str] = UNSET

    discount_code: OptionalNullable[str] = UNSET
    r"""Discount code to apply to the checkout."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "custom_field_data",
            "product_price_id",
            "amount",
            "customer_name",
            "customer_email",
            "customer_billing_address",
            "customer_tax_id",
            "discount_code",
        ]
        nullable_fields = [
            "custom_field_data",
            "product_price_id",
            "amount",
            "customer_name",
            "customer_email",
            "customer_billing_address",
            "customer_tax_id",
            "discount_code",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrantadsproperties import (
    BenefitGrantAdsProperties,
    BenefitGrantAdsPropertiesTypedDict,
)
from .benefitgrantcustomproperties import (
    BenefitGrantCustomProperties,
    BenefitGrantCustomPropertiesTypedDict,
)
from .benefitgrantdiscordproperties import (
    BenefitGrantDiscordProperties,
    BenefitGrantDiscordPropertiesTypedDict,
)
from .benefitgrantdownloadablesproperties import (
    BenefitGrantDownloadablesProperties,
    BenefitGrantDownloadablesPropertiesTypedDict,
)
from .benefitgrantgithubrepositoryproperties import (
    BenefitGrantGitHubRepositoryProperties,
    BenefitGrantGitHubRepositoryPropertiesTypedDict,
)
from .benefitgrantlicensekeysproperties import (
    BenefitGrantLicenseKeysProperties,
    BenefitGrantLicenseKeysPropertiesTypedDict,
)
from .customer import Customer, CustomerTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


PropertiesTypedDict = TypeAliasType(
    "PropertiesTypedDict",
    Union[
        BenefitGrantCustomPropertiesTypedDict,
        BenefitGrantDownloadablesPropertiesTypedDict,
        BenefitGrantAdsPropertiesTypedDict,
        BenefitGrantLicenseKeysPropertiesTypedDict,
        BenefitGrantDiscordPropertiesTypedDict,
        BenefitGrantGitHubRepositoryPropertiesTypedDict,
    ],
)


Properties = TypeAliasType(
    "Properties",
    Union[
        BenefitGrantCustomProperties,
        BenefitGrantDownloadablesProperties,
        BenefitGrantAdsProperties,
        BenefitGrantLicenseKeysProperties,
        BenefitGrantDiscordProperties,
        BenefitGrantGitHubRepositoryProperties,
    ],
)


class BenefitGrantTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the grant."""
    is_granted: bool
    r"""Whether the benefit is granted."""
    is_revoked: bool
    r"""Whether the benefit is revoked."""
    subscription_id: Nullable[str]
    r"""The ID of the subscription that granted this benefit."""
    order_id: Nullable[str]
    r"""The ID of the order that granted this benefit."""
    customer_id: str
    r"""The ID of the customer concerned by this grant."""
    user_id: str
    benefit_id: str
    r"""The ID of the benefit concerned by this grant."""
    customer: CustomerTypedDict
    r"""A customer in an organization."""
    properties: PropertiesTypedDict
    granted_at: NotRequired[Nullable[datetime]]
    r"""The timestamp when the benefit was granted. If `None`, the benefit is not granted."""
    revoked_at: NotRequired[Nullable[datetime]]
    r"""The timestamp when the benefit was revoked. If `None`, the benefit is not revoked."""


class BenefitGrant(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the grant."""

    is_granted: bool
    r"""Whether the benefit is granted."""

    is_revoked: bool
    r"""Whether the benefit is revoked."""

    subscription_id: Nullable[str]
    r"""The ID of the subscription that granted this benefit."""

    order_id: Nullable[str]
    r"""The ID of the order that granted this benefit."""

    customer_id: str
    r"""The ID of the customer concerned by this grant."""

    user_id: Annotated[
        str,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    benefit_id: str
    r"""The ID of the benefit concerned by this grant."""

    customer: Customer
    r"""A customer in an organization."""

    properties: Properties

    granted_at: OptionalNullable[datetime] = UNSET
    r"""The timestamp when the benefit was granted. If `None`, the benefit is not granted."""

    revoked_at: OptionalNullable[datetime] = UNSET
    r"""The timestamp when the benefit was revoked. If `None`, the benefit is not revoked."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["granted_at", "revoked_at"]
        nullable_fields = [
            "modified_at",
            "subscription_id",
            "order_id",
            "granted_at",
            "revoked_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .repository import Repository, RepositoryTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceRepositoryTypedDict(TypedDict):
    items: List[RepositoryTypedDict]
    pagination: PaginationTypedDict


class ListResourceRepository(BaseModel):
    items: List[Repository]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .advertisementcampaign import AdvertisementCampaign, AdvertisementCampaignTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class AdvertisementCampaignListResourceTypedDict(TypedDict):
    items: List[AdvertisementCampaignTypedDict]
    pagination: PaginationTypedDict
    dimensions: List[int]
    r"""The dimensions (width, height) in pixels of the advertisement images."""


class AdvertisementCampaignListResource(BaseModel):
    items: List[AdvertisementCampaign]

    pagination: Pagination

    dimensions: List[int]
    r"""The dimensions (width, height) in pixels of the advertisement images."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .oauth2client import OAuth2Client, OAuth2ClientTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceOAuth2ClientTypedDict(TypedDict):
    items: List[OAuth2ClientTypedDict]
    pagination: PaginationTypedDict


class ListResourceOAuth2Client(BaseModel):
    items: List[OAuth2Client]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrantwebhook import BenefitGrantWebhook, BenefitGrantWebhookTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookBenefitGrantUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a new benefit grant is updated.

    **Discord & Slack support:** Basic
    """

    data: BenefitGrantWebhookTypedDict
    type: Literal["benefit_grant.updated"]


class WebhookBenefitGrantUpdatedPayload(BaseModel):
    r"""Sent when a new benefit grant is updated.

    **Discord & Slack support:** Basic
    """

    data: BenefitGrantWebhook

    TYPE: Annotated[
        Annotated[
            Literal["benefit_grant.updated"],
            AfterValidator(validate_const("benefit_grant.updated")),
        ],
        pydantic.Field(alias="type"),
    ] = "benefit_grant.updated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrderBillingReason(str, Enum):
    PURCHASE = "purchase"
    SUBSCRIPTION_CREATE = "subscription_create"
    SUBSCRIPTION_CYCLE = "subscription_cycle"
    SUBSCRIPTION_UPDATE = "subscription_update"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ProductPriceRecurringFreeCreateTypedDict(TypedDict):
    r"""Schema to create a free recurring product price, i.e. a subscription."""

    recurring_interval: SubscriptionRecurringInterval
    type: Literal["recurring"]
    amount_type: Literal["free"]


class ProductPriceRecurringFreeCreate(BaseModel):
    r"""Schema to create a free recurring product price, i.e. a subscription."""

    recurring_interval: SubscriptionRecurringInterval

    TYPE: Annotated[
        Annotated[Literal["recurring"], AfterValidator(validate_const("recurring"))],
        pydantic.Field(alias="type"),
    ] = "recurring"

    AMOUNT_TYPE: Annotated[
        Annotated[Literal["free"], AfterValidator(validate_const("free"))],
        pydantic.Field(alias="amount_type"),
    ] = "free"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpricerecurringcustom import (
    ProductPriceRecurringCustom,
    ProductPriceRecurringCustomTypedDict,
)
from .productpricerecurringfixed import (
    ProductPriceRecurringFixed,
    ProductPriceRecurringFixedTypedDict,
)
from .productpricerecurringfree import (
    ProductPriceRecurringFree,
    ProductPriceRecurringFreeTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ProductPriceRecurringTypedDict = TypeAliasType(
    "ProductPriceRecurringTypedDict",
    Union[
        ProductPriceRecurringFreeTypedDict,
        ProductPriceRecurringFixedTypedDict,
        ProductPriceRecurringCustomTypedDict,
    ],
)


ProductPriceRecurring = TypeAliasType(
    "ProductPriceRecurring",
    Union[
        ProductPriceRecurringFree,
        ProductPriceRecurringFixed,
        ProductPriceRecurringCustom,
    ],
)
